package com.unissoft.intrusion;

import com.alibaba.fastjson.JSON;
import com.unissoft.common.DateUtils;
import com.unissoft.common.MyConstant;
import com.unissoft.utils.RedisUtil;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpMethod;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.util.StringUtils;
import org.springframework.web.client.RestTemplate;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @author qingsong.han
 * @description:
 * @create 2022-11-07 10:21
 */
public interface IntrusionService {
    void pushIntrusion(Integer sAlarmType, String alarmNow, String deviceIP, String orgCode);

    void editChannelCode(Map<String, String> channelCodes);

    @Data
    @Component
    class IntrusionModel {
        // 报警类型
        private Integer alarmType;
        // 报警时间
        private String alarmNow;
        // 设备ip
        private String deviceIP;
        // 机构编码
        private String orgCode;
        // ICC通道号
        private String channelCode;
    }

    // 定义内部类
    @Slf4j
    @Service
    class IntrusionServiceImpl implements IntrusionService {
        @Resource
        private RedisUtil redisUtil;
        /*@Resource
        private RestTemplate restTemplate;*/
        // 推送地址
        /*@Value("${address.pushToIntrusion}")
        private String pushToIntrusion;*/
        @Value("${spring.redis.ten-minutes}")
        private long tenMinutes;
        @Resource
        private IntrusionModel intrusionModel;

        @Override
        public void pushIntrusion(Integer sAlarmType, String alarmNow, String deviceIP, String orgCode) {
            /*// redis key
            if (redisUtil.hasKey(deviceIP)) {
                // key 存在丢弃报警数据,不上报
                log.info("Don't up!-> sAlarmType: {} alarmNow: {} deviceIP: {} orgCode: {}", sAlarmType, alarmNow, deviceIP, orgCode);
            } else {
                // 上报数据-接口
                log.info("Up it!-> sAlarmType: {} alarmNow: {} deviceIP: {} orgCode: {}", sAlarmType, alarmNow, deviceIP, orgCode);
                intrusionModel.setDeviceIP(deviceIP);
                intrusionModel.setSAlarmType(sAlarmType);
                intrusionModel.setAlarmNow(alarmNow);
                intrusionModel.setOrgCode(orgCode);
                HttpEntity<IntrusionModel> httpEntity = new HttpEntity<>(intrusionModel);
                restTemplate.exchange(pushToIntrusion, HttpMethod.POST, httpEntity, Object.class);
                // 设置缓存
                redisUtil.set(deviceIP, JSON.toJSONString(intrusionModel), tenMinutes);
            }*/
            // TODO 物联网到业务平台推送数据网络不通,修改为业务平台主动获取数据
            // 上报数据-接口

            intrusionModel.setDeviceIP(deviceIP);
            intrusionModel.setAlarmType(sAlarmType);
            intrusionModel.setAlarmNow(alarmNow);
            intrusionModel.setOrgCode(orgCode);
            // 添加通道号
            intrusionModel.setChannelCode(this.getChannelCode(deviceIP));
            // 设置异常闯入设备数据-hashCode
            redisUtil.hset(MyConstant.INTRUSION_ALARM_IP, deviceIP, intrusionModel, tenMinutes);
            log.info("Up it!-> sAlarmType: {} alarmNow: {} deviceIP: {} orgCode: {}", sAlarmType, alarmNow, deviceIP, orgCode);
        }

        private String getChannelCode(String deviceIP) {
            String channelCode = redisUtil.hget(MyConstant.ICC_CHANNEL_CODE, deviceIP).toString();
            if (StringUtils.isEmpty(channelCode))
                channelCode = this.staticChannelCode(deviceIP);
            return channelCode;
        }

        private String staticChannelCode(String deviceIP) {
            switch (deviceIP) {
                case "192.168.193.16":
                    return "1000384$1$0$10";
                case "192.168.193.17":
                    return "1000384$1$0$8";
                case "192.168.143.10":
                    return "1000365$1$0$0";
                case "192.168.143.11":
                    return "1000365$1$0$1";
                case "192.168.163.21":
                    return "1000445$1$0$25";
                case "192.168.153.15":
                    return "1000462$1$0$6";
                case "192.168.153.16":
                    return "1000462$1$0$5";
                case "192.168.153.20":
                    return "1000462$1$0$10";
                case "192.168.153.22":
                    return "1000462$1$0$12";
                case "192.168.153.23":
                    return "1000462$1$0$13";
                case "192.168.153.25":
                    return "1000462$1$0$15";
                case "192.168.173.162":
                    return "1000221$1$0$0";
                case "192.168.173.163":
                    return "1000220$1$0$0";
                case "192.168.173.164":
                    return "1000200$1$0$0";
                default:
                    return "";
            }
        }

        @Override
        public void editChannelCode(Map<String, String> channelCodes) {
            channelCodes.forEach((ip, code) ->
                    redisUtil.hset(MyConstant.ICC_CHANNEL_CODE, ip, code)
            );
        }

    }


}
